﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GaCa
{
	class ES
	{
		MultiVariableFitness fitness;
		//VariableFitness fitness;
		//StateFitness fitness;
		//GraphFitness fitness;
		Development development;

		public CA BestCA { get; set; }
		public Phenotype BestPhenotype { get; set; }
		public double BestFitness { get; set; }
		public string BestFinalState { get; set; }
		public int Generation { get; set; }

		public CASettings CASettings { get; set; }
		public ESSettings Settings { get; set; }

		public List<double> F1 { get; set; }
		public List<double> F2 { get; set; }
		public List<double> F3 { get; set; }
		public List<double> F4 { get; set; }
		public List<double> F5 { get; set; }

		public ES(CASettings caSettings, ESSettings esSettings)
		{
			this.CASettings = caSettings;
			this.Settings = esSettings;

			this.development = new Development();
			//this.fitness = new GraphFitness(esSettings.TargetVariables, esSettings.FitnessType);
			//this.fitness = new GraphFitness(esSettings.TargetGraph);
			//this.fitness = new StateFitness(Settings);
			//this.fitness = new VariableFitness(esSettings.TargetVariables, esSettings.FitnessType);
			this.fitness = new MultiVariableFitness(esSettings.T1, esSettings.T2, esSettings.T3, esSettings.T4, esSettings.T5);

			this.BestCA = new CA(caSettings);

			//this.BestPhenotype = development.Develop(this.BestCA, esSettings.InitialState);
			//this.BestFitness = fitness.Calculate(this.BestPhenotype.HalfMagnitudes);
			//this.BestFitness = fitness.Calculate(this.BestPhenotype.Variables);
			//this.BestFitness = fitness.Calculate(this.BestCA);
			List<double> v1 = development.Develop(this.BestCA, esSettings.I1).Variables;
			List<double> v2 = development.Develop(this.BestCA, esSettings.I2).Variables;
			List<double> v3 = development.Develop(this.BestCA, esSettings.I3).Variables;
			List<double> v4 = development.Develop(this.BestCA, esSettings.I4).Variables;
			List<double> v5 = development.Develop(this.BestCA, esSettings.I5).Variables;
			this.BestFitness = fitness.Calculate(v1, v2, v3, v4, v5);
			this.Generation = 0;
		}

		//public void Reset(List<double> targetVariables)
		//{
		//    //this.fitness = new SpikeFitness(targetVariables, ESSettings.FitnessType);
		//    //this.fitness = new StateFitness(ESSettings.InitialState, ESSettings.FinalState, ESSettings.DevelopmentSteps);
		//    this.BestFitness = 0;
		//    this.Generation = 0;
		//}

		public void Reset()
		{
			Reset(Settings);
		}

		public void Reset(ESSettings esSettings)
		{
			this.Settings = esSettings;
			//this.fitness = new StateFitness(esSettings);
			//this.fitness = new GraphFitness(esSettings.TargetGraph);
			//this.fitness = new VariableFitness(esSettings.TargetVariables, esSettings.FitnessType);

			this.fitness = new MultiVariableFitness(esSettings.T1, esSettings.T2, esSettings.T3, esSettings.T4, esSettings.T5);

			this.BestCA = new CA(CASettings);

			List<double> v1 = development.Develop(this.BestCA, esSettings.I1).Variables; 
			List<double> v2 = development.Develop(this.BestCA, esSettings.I2).Variables; 
			List<double> v3 = development.Develop(this.BestCA, esSettings.I3).Variables; 
			List<double> v4 = development.Develop(this.BestCA, esSettings.I4).Variables; 
			List<double> v5 = development.Develop(this.BestCA, esSettings.I5).Variables; 
			this.BestFitness = fitness.Calculate(v1, v2, v3, v4, v5);

			this.BestFitness = 0;
			this.Generation = 0;
		}

		public void Step()
		{
			//Console.WriteLine("G{0}: {1} ({2})", Generation, BestCA, BestFitness); 
			List<CA> clones = GetClones(this.BestCA);
			UpdateWinner(clones);
			this.Generation++;
		}

		private List<CA> GetClones(CA origin)
		{
			List<CA> clones = new List<CA>();
			for (int i = 0; i < Settings.ClonesPerCandidate; i++)
			{
				CA clone = Mutate(origin);
				clones.Add(clone);
			}
			return clones;
		}

		private void UpdateWinner(List<CA> clones)
		{
			//double tempFitness = 0;
			//CA tempClone = null;
			foreach (CA clone in clones)
			{
				Phenotype clonePhenotype = development.Develop(clone, Settings.InitialState);
				//double cloneFitness = fitness.Calculate(clonePhenotype.HalfMagnitudes);
				//double cloneFitness = fitness.Calculate(clonePhenotype.Variables);

				List<double> v1 = development.Develop(clone, Settings.I1).Variables;
				List<double> v2 = development.Develop(clone, Settings.I2).Variables;
				List<double> v3 = development.Develop(clone, Settings.I3).Variables;
				List<double> v4 = development.Develop(clone, Settings.I4).Variables;
				List<double> v5 = development.Develop(clone, Settings.I5).Variables;
				double cloneFitness = fitness.Calculate(v1, v2, v3, v4, v5);



				//double cloneFitness = fitness.Calculate(clone);
				//Console.WriteLine("clone: {0} ({1})", clone, cloneFitness);
				if (cloneFitness >= this.BestFitness)
				//if (cloneFitness > tempFitness)
				{
					F1 = v1;
					F2 = v2;
					F3 = v3;
					F4 = v4;
					F5 = v5;
					//tempClone = clone;
					this.BestCA = clone;
					//this.BestPhenotype = clonePhenotype;
					//tempFitness = cloneFitness;
					this.BestFitness = cloneFitness;
					//this.BestFinalState = 
				}
			}
			//this.BestCA = tempClone;
			//this.BestFitness = tempFitness;
		}

		private CA Mutate(CA ca)
		{
			StringBuilder sb = new StringBuilder(ca.Rule);
			for (int i = 0; i < ca.Rule.Length; i++)
			{
				if (Tools.Random.NextDouble() < Settings.MutationRate)
					sb[i] = sb[i].Flip(CASettings.StatesPerCell);
			}
			CA mutant = new CA(sb.ToString(), ca.Settings);
			//mutant.Rule = sb.ToString();
			return mutant;
		}
	}

}
