using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using AForge.Math;
using System.Text;

namespace GaCa
{
	/// <summary>
	/// This is the main type for your game
	/// </summary>
	public class EvolutionGame : Microsoft.Xna.Framework.Game
	{
		GraphicsDeviceManager graphics;
		SpriteBatch spriteBatch;
		SpriteFont font;
		Texture2D square;
		Input input;
		//Development development;

		int WaitTime { get; set; }
		//public bool Running { get; set; }

		//double bestFitness;
		//int currentStep;
		//ES es;
		Phenotype targetPhenotype;
		List<double> targetVariables;
		//Phenotype bestPhenotype;

		//CASettings caSettings;
		ESSettings esSettings;

		Writer writer;
		Menu caMenu;

		CA targetCA;

		string init;

		Experiment experiment;

		public EvolutionGame()
		{
			graphics = new GraphicsDeviceManager(this);
			Content.RootDirectory = "Content";
			input = new Input();

			CASettings caSettings = new CASettings
			{
				Boundary = Development.Boundary.Cyclic,
				Interpretation = Development.Interpretation.Decimal,
				Neighborhood = 3,
				Size = 9,
				//Size = 31,
				StatesPerCell = 3,
				//StatesPerCell = 2,
				Threshold = 50,
				DevelopmentType = Development.Type.TransientOnePeriod,
				Steps = 64,
			};

			//init = "0101010101010101010101010101010";
			init = "010101010";
			//init = "010101010101010101010";

			caMenu = new Menu(caSettings);
			targetCA = caSettings.GenerateCA();

			//init = caSettings.GenerateState();

			//targetPhenotype = GetTargetPhenotype();

			//targetVariables = GetTargetVariables(3, 2);

			
			//this.targetCA = caSettings.GenerateCA();
			//targetCA.Rule = Tools.DecimalToBase(97, 2, 8);

			//Development dev = new Development(targetCA);
			//Phenotype phen = dev.Develop(targetCA, init);

			//Console.WriteLine("vars: " + phen.Variables.Serialize());

			esSettings = new ESSettings
			{
				ClonesPerCandidate = 4,
				MutationRate = 0.35,
				//TargetGraph = phen.HalfMagnitudes,
				//TargetVariables = phen.Variables,
				TargetVariables = GetTargetVariables(3, 2),
				FitnessType = VariableFitness.Comparison.StrictCount,
				InitialState = init,
				FinalState = Final(targetCA, init, 64),
				DevelopmentSteps = 64,
				MaxGenerations = 1000000
			};

			esSettings.I1 = "102001001";
			esSettings.I2 = "120010201";
			esSettings.I3 = "222010020";
			esSettings.I4 = "100220110";
			esSettings.I5 = "201121101";

			esSettings.T1 = new double[] { 0.45, 1.11, 0.86 }.ToList();
			esSettings.T2 = new double[] { 0.11, 0.99, 1.55 }.ToList();
			esSettings.T3 = new double[] { 0.73, 0.39, 0.91 }.ToList();
			esSettings.T4 = new double[] { 1.21, 0.23, 0.41 }.ToList();
			esSettings.T5 = new double[] { 0.87, 1.04, 0.76 }.ToList();

			Console.WriteLine("rand vars: " + esSettings.TargetVariables.Serialize());

			

			//InitTarget();
			
			//dummyInit = "0110111000110011";
			//currentStep = 0;
			WaitTime = 1000;
			//bestFitness = 0;

			//es = new ES(caSettings, esSettings);
			//{
			//    ClonesPerCandidate = 4,
			//    MutationRate = 0.15,
			//    TargetVariables = targetPhenotype.Variables,
			//};
			//es.Init();
			//Reset();
		}

		/// <summary>
		/// Allows the game to perform any initialization it needs to before starting to run.
		/// This is where it can query for any required services and load any non-graphic
		/// related content.  Calling base.Initialize will enumerate through any components
		/// and initialize them as well.
		/// </summary>
		protected override void Initialize()
		{
			// TODO: Add your initialization logic here
			this.IsMouseVisible = true;
			base.Initialize();
			LineBatch.Init(GraphicsDevice);
		}

		/// <summary>
		/// LoadContent will be called once per game and is the place to load
		/// all of your content.
		/// </summary>
		protected override void LoadContent()
		{
			spriteBatch = new SpriteBatch(GraphicsDevice);

			font = Content.Load<SpriteFont>("font1");
			square = new Texture2D(GraphicsDevice, 1, 1);
			square.SetData(new[] { Color.White });
			writer = new Writer(spriteBatch, font);
		}

		/// <summary>
		/// UnloadContent will be called once per game and is the place to unload
		/// all content.
		/// </summary>
		protected override void UnloadContent()
		{
			// TODO: Unload any non ContentManager content here
		}

		/// <summary>
		/// Allows the game to run logic such as updating the world,
		/// checking for collisions, gathering input, and playing audio.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Update(GameTime gameTime)
		{

			if (gameTime.TotalGameTime.Milliseconds % WaitTime == 0 && experiment != null && experiment.State == Experiment.States.Running)
			{
				//Console.WriteLine("This should happen. G{0}", es.Generation);
				//es.Step();
				//if (es.BestFitness == 1) Running = false;
				//currentStep++;
				//this.bestPhenotype = es.BestPhenotype;
				//this.bestFitness = es.BestFitness;
				experiment.Step();
			}

			input.Load();

			if (input.Pressed(Input.Inputs.Right))
			{
				caMenu.Next();
			}

			if (input.Pressed(Input.Inputs.Left))
			{
				caMenu.Prev();
			}

			if (input.Pressed(Input.Inputs.Down))
			{
				caMenu.Decrease();
				InitTarget();
			}

			if (input.Pressed(Input.Inputs.Up))
			{
				caMenu.Increase();
				InitTarget();
			}

			if (input.Pressed(Input.Inputs.R))
			{
				InitTarget();
			}

			if (input.Pressed(Input.Inputs.F))
			{
				caMenu.Settings.StatesPerCell = 3;
				//string init = "0000000000000001000000000000000";
				//string init = "00000100000";
				string init = "0001000";
				caMenu.Settings.Size = init.Length;
				CA ca = new CA(caMenu.Settings);
				ca.Rule = "120121002102100222210022201";
				string final = Final(ca, init, 64);
				Console.WriteLine("CA: {0}\nFinal: {1}", ca.Rule, final);
				esSettings.FinalState = final;
				esSettings.InitialState = init;
				esSettings.DevelopmentSteps = 64;
			}

			if (input.Pressed(Input.Inputs.Space))
			{
				ES es = new ES(caMenu.Settings, esSettings);
				experiment = new Experiment(es, 1, 1);
				//es = new ES(caMenu.Settings, esSettings);
				//es.CASettings = caMenu.Settings;
				//es.Reset(esSettings);
			}

			if (input.Pressed(Input.Inputs.D))
			{
				Console.WriteLine("Winners: ");
				foreach (string rule in experiment.FinalRules)
				{
					Console.Write("{0} ({1}), ", rule, Tools.BaseToDecimal(rule, caMenu.Settings.StatesPerCell));
				}
			}

			input.Save();

			base.Update(gameTime);
		}

		/// <summary>
		/// This is called when the game should draw itself.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Draw(GameTime gameTime)
		{
			GraphicsDevice.Clear(Color.CornflowerBlue);

			spriteBatch.Begin();
			writer.Begin();

			int padding = 10;
			int offset = padding;
			foreach (KeyValuePair<CASettings.Setting, string> kvPair in caMenu.Settings.Dictionary())
			{
				writer.Y = 30;
				writer.Write(true, kvPair.Value);
				writer.Y = 10;
				writer.Write(caMenu.Current == kvPair.Key ? Color.Red : Color.White, kvPair.Key.ToString());
			}

			writer.WriteLine();
			writer.WriteLine();

			writer.WriteLine("Initial: {0}", esSettings.InitialState);
			writer.WriteLine("Final: {0}", esSettings.FinalState);
			writer.WriteLine("Steps: {0}", esSettings.DevelopmentSteps);

			if (experiment != null)
			{
				writer.Write("N: {0}", experiment.CurrentN);
				writer.Write("Winner: {0}", experiment.CurrentRule);
				writer.Write("Generations: {0}", experiment.CurrentGenCount);
			}

			if (experiment != null && experiment.State == Experiment.States.Finished)
			{
				writer.WriteLine();
				//writer.Console(true);
				writer.WriteLine("Mean: {0}", Math.Round(experiment.FinalMean, 3));
				//writer.WriteLine("Variance: {0}", Math.Round(experiment.FinalVariance));
				writer.WriteLine("Standard deviation: {0}", Math.Round(experiment.FinalDeviation, 3));
				writer.WriteLine("Min: {0}", experiment.FinalMin);
				writer.WriteLine("Max: {0}", experiment.FinalMax);

				writer.Write("Winners: {0}", experiment.FinalRules.Count);
				/*foreach (string rule in experiment.FinalRules)
				{
					writer.Write("{0} ({1}), ", rule, Tools.BaseToDecimal(rule, caMenu.Settings.StatesPerCell));
				}*/
				//writer.Console(false);
			}

			spriteBatch.End();

			base.Draw(gameTime);
		}

		Phenotype GetTargetPhenotype()
		{
			CA ca = new CA(caMenu.Settings);
			Development development = new Development(ca);
			Phenotype phenotype = development.Develop(ca, this.init);
			return phenotype;
		}

		List<double> GetTargetVariables(int count, int decimals)
		{
			List<double> variables = new List<double>();
			variables.Add(0.21);
			variables.Add(0.73);
			variables.Add(0.78);
			//for (int i = 0; i < count; i++)
				//variables.Add(Math.Round(Tools.Random.NextDouble(), decimals));
			return variables;
		}

		string Final(CA ca, string init, int steps)
		{
			Development development = new Development(ca);
			return development.Next(init, steps);
		}

		void InitTarget()
		{
			this.targetCA = caMenu.Settings.GenerateCA();
			//esSettings.InitialState = caMenu.Settings.GenerateState();
			//esSettings.FinalState = Final(targetCA, esSettings.InitialState, esSettings.DevelopmentSteps);

			esSettings.InitialState = "00000000000000000000000000000000100000000000000000000000000000000";
			esSettings.FinalState = "11111111111111111111111111111111011111111111111111111111111111111";
			esSettings.DevelopmentSteps = 64;
			caMenu.Settings.Size = esSettings.InitialState.Length;

			//targetPhenotype = GetTargetPhenotype();
			//this.targetVariables = GetTargetVariables(3, 2);
			//es.Reset(targetPhenotype.Variables);
			//es.Reset(esSettings);
		}
	}
}
