﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GaCa
{
	class Experiment
	{
		ES es;
		int N;
		double requiredFitness;
		List<int> genCounts;

		public enum States { Running, Finished };

		public States State { get; set; }

		public int CurrentN { get; set; }
		public string CurrentRule { get; set; }
		public int CurrentGenCount { get; set; }

		public List<string> FinalRules { get; set; }
		public double FinalMean { get; set; }
		public double FinalVariance { get; set; }
		public double FinalDeviation { get; set; }
		public int FinalMin { get; set; }
		public int FinalMax { get; set; }

		List<double> Fitnesses { get; set; }
		public List<double> FinalFitnesses { get; set; }

		//int factor;

		public Experiment(ES es, int N, double requiredFitness)
		{
			this.N = N;
			this.es = es;
			this.requiredFitness = requiredFitness;
			this.CurrentN = 0;
			this.genCounts = new List<int>();
			this.State = States.Running;
			this.FinalRules = new List<string>();
			this.Fitnesses = new List<double>();
			this.FinalFitnesses = new List<double>();
		}

		public void Step()
		{
			es.Reset();
			int i = 0;
			Console.Write("{0}\t", es.BestFitness);
			while (es.BestFitness < requiredFitness && es.Generation < es.Settings.MaxGenerations)
			{
				es.Step();
				
				i++;
				Fitnesses.Add(es.BestFitness);
				if (i % 500 == 0)
				{
					//Fitnesses.Add(es.BestFitness);
					//Console.Write("{0}\t", i);
					Console.Write("{0}\t", es.BestFitness);
				}
			}
			Console.Write(es.BestFitness + "\n");
			CurrentRule = es.BestCA.Rule;
			CurrentGenCount = es.Generation;
			
			genCounts.Add(CurrentGenCount);
			if (es.BestFitness >= requiredFitness)
			{
				if (FinalRules.IndexOf(CurrentRule) == -1)
					FinalRules.Add(CurrentRule);
			}

			CurrentN++;
			if (CurrentN >= N)
			{
				State = States.Finished;
				Calculate();
				//Console.Write("{0}\t", es.BestFitness);
				Console.WriteLine("V1: " + es.F1.Serialize());
				Console.WriteLine("V2: " + es.F2.Serialize());
				Console.WriteLine("V3: " + es.F3.Serialize());
				Console.WriteLine("V4: " + es.F4.Serialize());
				Console.WriteLine("V5: " + es.F5.Serialize());
				return;
			}

		}

		void Calculate()
		{
			double sum = 0;
			foreach (int gen in genCounts)
				sum += gen;
			FinalMean = sum / genCounts.Count;

			sum = 0;
			foreach (int gen in genCounts)
			{
				double diff = gen - FinalMean;
				sum += diff * diff;
			}
			FinalDeviation = sum / genCounts.Count;

			FinalVariance = FinalDeviation * FinalDeviation;
			FinalMin = genCounts.Min();
			FinalMax = genCounts.Max();
			FinalRules.Sort();

			if (Fitnesses.Count > 100)
				FinalFitnesses = Fitnesses.GetRange(Fitnesses.Count - 100, 100);
			else
				FinalFitnesses = Fitnesses;

			//int factor = 1000 / Fitnesses.Count;
			//Console.WriteLine("Fitnesses:");

			//Console.WriteLine("\nLast:");
			//for (int i = 0; i < FinalFitnesses.Count; i++)
			//{
			//    Console.Write(FinalFitnesses[i] + "\t");
			//}

		}
	}
}
