using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using AForge.Math;
using System.Text;

namespace GaCa
{
	/// <summary>
	/// This is the main type for your game
	/// </summary>
	public class ExplorationGame : Microsoft.Xna.Framework.Game
	{
		GraphicsDeviceManager graphics;
		SpriteBatch spriteBatch;
		//CA ca;
		SpriteFont font;
		Texture2D square;
		Input input;
		
		public Size CellSize { get; set; }
		public int CellPadding { get; set; }
		public int WaitTime { get; set; }
		public bool Running { get; set; }
		public bool SuperSpeed { get; set; }

        public List<string> states;

		Development development;

		CASettings settings;
		string state;
		int steps;
		int step;

		public ExplorationGame()
		{
			graphics = new GraphicsDeviceManager(this);
			Content.RootDirectory = "Content";
			input = new Input();

			WaitTime = 100;
			Running = false;

			settings = new CASettings
			{
				Boundary = Development.Boundary.Cyclic,
				Neighborhood = 3,
				Size = 65,
				StatesPerCell = 2,
			};
			
			step = 1;
			Console.WriteLine("Starting on step " + step);
			steps = 64;
			state = "0101010101010101010101010101010";
			settings.Size = state.Length;
			states = new List<string>();
			states.Add(state);
			string rule = "01101110";
			CA ca = new CA(rule, settings);
			development = new Development(ca);

			SuperSpeed = false;
			CellPadding = 2;
			CellSize = new Size(22, 22);

            
			
			//0AB8V5
		}

		/// <summary>
		/// Allows the game to perform any initialization it needs to before starting to run.
		/// This is where it can query for any required services and load any non-graphic
		/// related content.  Calling base.Initialize will enumerate through any components
		/// and initialize them as well.
		/// </summary>
		protected override void Initialize()
		{
			// TODO: Add your initialization logic here
			this.IsMouseVisible = true;
			base.Initialize();
			LineBatch.Init(GraphicsDevice);

			//this.ca = new CA(new Settings());
			//StringBuilder sb = new StringBuilder();
			//for (int i = 0; i < settings.Size; i++)
			//	sb.Append(Tools.Random.Next(2));
			//string init = sb.ToString();
			//ca.Initialize("00000000000000000000000000100000000000000000000000");
			//ca.Rule = Convert.ToString(22, 2).PadLeft(8, '0');
			//states.Add(ca.GetState());
		}

		/// <summary>
		/// LoadContent will be called once per game and is the place to load
		/// all of your content.
		/// </summary>
		protected override void LoadContent()
		{
			// Create a new SpriteBatch, which can be used to draw textures.
			spriteBatch = new SpriteBatch(GraphicsDevice);

			font = Content.Load<SpriteFont>("font1");
			square = new Texture2D(GraphicsDevice, 1, 1);
			square.SetData(new[] { Color.White });
		}

		/// <summary>
		/// UnloadContent will be called once per game and is the place to unload
		/// all content.
		/// </summary>
		protected override void UnloadContent()
		{
			// TODO: Unload any non ContentManager content here
		}

		/// <summary>
		/// Allows the game to run logic such as updating the world,
		/// checking for collisions, gathering input, and playing audio.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Update(GameTime gameTime)
		{

			if (Running)
			{
				if (
					Keyboard.GetState().IsKeyDown(Keys.LeftShift) ||
					SuperSpeed ||
					gameTime.TotalGameTime.Milliseconds % WaitTime == 0
				)
				{
					step++;
					Console.WriteLine("Doing step " + step);

					string next = development.Next(state);
					states.Add(next);
					state = next;

					if (step >= steps) Running = false;
					
					//ca.NextState();
                    //states.Add(ca.GetState());
                    //Console.WriteLine(ca.GetState());
				}
			}

			input.Load();

			if (input.Pressed(Input.Inputs.Space))
				Running = !Running;

            if (input.Pressed(Input.Inputs.R))
            {
				//this.ca.Randomize();
                states.Clear();
                //states.Add(ca.GetState());
            }

			if (input.Pressed(Input.Inputs.Caps))
				SuperSpeed = !SuperSpeed;

			if (input.Pressed(Input.Inputs.Escape))
				this.Exit();

			input.Save();

			base.Update(gameTime);
		}

		/// <summary>
		/// This is called when the game should draw itself.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Draw(GameTime gameTime)
		{
			Resize();

			GraphicsDevice.Clear(Color.CornflowerBlue);

			spriteBatch.Begin();

			for (int y = 0; y < states.Count; y++)
			{
                string state = states[y];
				for (int x = 0; x < settings.Size; x++)
				{
                    char signal = state[x];
					Color color = (signal == '1' ? Color.Black : Color.White);
					Rectangle rect = new Rectangle(
						CellPadding + x * (CellSize.Width + CellPadding),
						CellPadding + y * (CellSize.Height + CellPadding),
						CellSize.Width,
						CellSize.Height
					);
					spriteBatch.Draw(square, rect, color);
				}
			}

			////string status = (Running ? "Running" : "Paused");
			////spriteBatch.DrawString(font, status, new Vector2(0, 440), Color.White);

			spriteBatch.End();

			base.Draw(gameTime);
		}

        //Position GetCellPos(Position mousePos)
        //{
        //    int x = (mousePos.X - CellPadding) / (CellSize.Width + CellPadding);
        //    int y = (mousePos.Y - CellPadding) / (CellSize.Height + CellPadding);
        //    if (x > -1 && x < ca.GetSize().Width && y > -1 && y < ca.GetSize().Height)
        //        return new Position(x, y);
        //    return null;
        //}

		public void Resize()
		{
			graphics.PreferredBackBufferWidth = settings.Size * (CellSize.Width + CellPadding) + CellPadding;
            graphics.PreferredBackBufferHeight = states.Count * (CellSize.Height + CellPadding) + CellPadding;
			graphics.ApplyChanges();
		}
	}
}
