using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using AForge.Math;
using System.Text;

namespace GaCa
{
	/// <summary>
	/// This is the main type for your game
	/// </summary>
	public class SettingsGame : Microsoft.Xna.Framework.Game
	{
		GraphicsDeviceManager graphics;
		SpriteBatch spriteBatch;
		SpriteFont font;
		Texture2D square;
		Input input;
		Phenotype phenotype;
		//Generator generator;
		//CASettings settings;
		Development development;
		CA ca;
		Menu menu;
		string init;

		CASettings.Setting Setting { get; set; }

		public SettingsGame()
		{
			graphics = new GraphicsDeviceManager(this);
			graphics.PreferredBackBufferHeight = 600;
			graphics.PreferredBackBufferWidth = 1000;
			Content.RootDirectory = "Content";
			input = new Input();

			CASettings settings = new CASettings
			{
				Boundary = Development.Boundary.Cyclic,
				Interpretation = Development.Interpretation.Sum,
				Neighborhood = 3,
				Size = 5,
				StatesPerCell = 3,
				Periods = 10,
				DevelopmentType = Development.Type.NSteps,
				Steps = 64,
			};

			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < settings.Size; i++)
				sb.Append(i % settings.StatesPerCell);
			init = sb.ToString();
			//init = "01010";

			ca = new CA(settings);
			menu = new Menu(settings);
			development = new Development();
			phenotype = development.Develop(ca, init);
		}

		/// <summary>
		/// Allows the game to perform any initialization it needs to before starting to run.
		/// This is where it can query for any required services and load any non-graphic
		/// related content.  Calling base.Initialize will enumerate through any components
		/// and initialize them as well.
		/// </summary>
		protected override void Initialize()
		{
			this.IsMouseVisible = true;
			base.Initialize();
			LineBatch.Init(GraphicsDevice);

			//ca.Initialize(settings);
			
		}

		/// <summary>
		/// LoadContent will be called once per game and is the place to load
		/// all of your content.
		/// </summary>
		protected override void LoadContent()
		{
			spriteBatch = new SpriteBatch(GraphicsDevice);
			font = Content.Load<SpriteFont>("font1");
			square = new Texture2D(GraphicsDevice, 1, 1);
			square.SetData(new[] { Color.White });
		}

		/// <summary>
		/// UnloadContent will be called once per game and is the place to unload
		/// all content.
		/// </summary>
		protected override void UnloadContent()
		{
			// TODO: Unload any non ContentManager content here
		}

		/// <summary>
		/// Allows the game to run logic such as updating the world,
		/// checking for collisions, gathering input, and playing audio.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Update(GameTime gameTime)
		{

			input.Load();

			if (input.Pressed(Input.Inputs.Space))
			{
				//ca.Settings = menu.Settings;
				ca = ca.Settings.GenerateCA();
				//ca.Rule = Tools.DecimalToBase(41, 2, 8);
				phenotype = development.Develop(ca, init);
			}

			if (input.Pressed(Input.Inputs.R))
			{
				menu.Settings = new CASettings();
				ca.Settings = menu.Settings;
				phenotype = development.Develop(ca, init);
			}

			if (input.Pressed(Input.Inputs.One))
			{
				//menu.Settings
				//ca.Settings = menu.Settings;
				ca.Rule = Tools.DecimalToBase(100, 2, 8);
				phenotype = development.Develop(ca, init);
			}

			if (input.Pressed(Input.Inputs.Two))
			{
				//menu.Settings = new CASettings(2);
				//ca.Settings = menu.Settings;
				ca.Rule = Tools.DecimalToBase(112, 2, 8);
				phenotype = development.Develop(ca, init);
			}

			if (input.Pressed(Input.Inputs.Three))
			{
				//menu.Settings = new CASettings(3);
				//ca.Settings = menu.Settings;
				ca.Rule = Tools.DecimalToBase(7, 2, 8);
				phenotype = development.Develop(ca, init);
			}

			if (input.Pressed(Input.Inputs.Four))
			{
				//menu.Settings = new CASettings(3);
				//ca.Settings = menu.Settings;
				ca.Rule = Tools.DecimalToBase(90, 2, 8);
				phenotype = development.Develop(ca, init);
			}

			if (input.Pressed(Input.Inputs.Five))
			{
				//menu.Settings = new CASettings(3);
				//ca.Settings = menu.Settings;
				ca.Rule = Tools.DecimalToBase(30, 2, 8);
				phenotype = development.Develop(ca, init);
			}

			if (input.Pressed(Input.Inputs.Six))
			{
				//menu.Settings = new CASettings(3);
				//ca.Settings = menu.Settings;
				ca.Rule = Tools.DecimalToBase(110, 2, 8);
				phenotype = development.Develop(ca, init);
			}

			if (input.Pressed(Input.Inputs.Right))
			{
				menu.Next();
			}

			if (input.Pressed(Input.Inputs.Left))
			{
				menu.Prev();
			}

			if (input.Pressed(Input.Inputs.Down))
			{
				menu.Decrease();
				ca.Settings = menu.Settings;
				if (init.Length > ca.Settings.Size)
					init = init.Substring(0, ca.Settings.Size);
				phenotype = development.Develop(ca, init);
			}

			if (input.Pressed(Input.Inputs.Up))
			{
				menu.Increase();
				ca.Settings = menu.Settings;
				if (init.Length < ca.Settings.Size)
					init += init.Length % 2 == 1 ? "1" : "0";
				phenotype = development.Develop(ca, init);
			}

			input.Save();

			base.Update(gameTime);
		}

		/// <summary>
		/// This is called when the game should draw itself.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Draw(GameTime gameTime)
		{
			GraphicsDevice.Clear(Color.White);

			spriteBatch.Begin();

			// behavior, preperiodic
			for (int x = 0; x < phenotype.Genotype.Settings.Size; x++)
			{
				for (int y = 0; y < phenotype.States.Count; y++)
				{
					int signal = (int)Char.GetNumericValue(phenotype.States[y][x]);
					int colorValue = 255 - (signal) * 255 / (menu.Settings.StatesPerCell - 1);
					Color color = new Color(colorValue, colorValue, colorValue);
					spriteBatch.DrawPoint(color, new Vector2(10 + y, 100 + (phenotype.Genotype.Settings.Size - x)));
				}
			}

			// behavior, periodic
			/*for (int j = 0; j < menu.Settings.Periods; j++)
			{
				for (int x = 0; x < phenotype.Genotype.Settings.Size; x++)
				{
					for (int y = 0; y < phenotype.PeriodicLength; y++)
					{
						int signal = (int)Char.GetNumericValue(phenotype.States[phenotype.PrePeriodicLength + y][x]);
						int colorValue = signal * 255 / (menu.Settings.StatesPerCell - 1);
						Color color = new Color(colorValue, colorValue, colorValue);
						spriteBatch.DrawPoint(color, new Vector2(10 + y + j * phenotype.PeriodicLength + phenotype.PrePeriodicLength, 100 + x));
					}
				}
			}*/

			// interpretation, pre-periodic
			for (int x = 0; x < phenotype.States.Count; x++)
				spriteBatch.DrawLine(Color.Black, new Vector2(10 + x, 260 + menu.Settings.Size), new Vector2(10 + x, 260 + menu.Settings.Size - phenotype.Interpretations[x] / 64));

			// interpretation, periodic
			/*for (int j = 0; j < menu.Settings.Periods; j++)
			{
				for (int x = 0; x < phenotype.PeriodicLength; x++)
				{
					int xOffset = 10 + phenotype.PrePeriodicLength + j * phenotype.PeriodicLength;
					spriteBatch.DrawLine(Color.Black, new Vector2(xOffset + x, 160 + menu.Settings.Size), new Vector2(xOffset + x, 160 + menu.Settings.Size - phenotype.PeriodicPath[x] / 32));
				}
			}*/

			// transformation
			int max = 500;
			for (int x = 0; x < phenotype.Magnitudes.Count; x++)
			{
				int top = 590 - phenotype.Magnitudes[x] / 2;
				if (top < max) top = max;
				spriteBatch.DrawLine(Color.Black, new Vector2(10 + x, 590), new Vector2(10 + x, top));
			}
			//spriteBatch.DrawLine(Color.Red, new Vector2(10, 450 - menu.Settings.Threshold), new Vector2(phenotype.Magnitudes.Count / 2 + 10, 450 - menu.Settings.Threshold));

			spriteBatch.DrawString(font, String.Format("Initial: {0}", init), new Vector2(10, 10), Color.Black);
			spriteBatch.DrawString(font, String.Format("Rule: {0} ({1})", phenotype.Genotype.Rule, Tools.BaseToDecimal(phenotype.Genotype.Rule, menu.Settings.StatesPerCell)), new Vector2(10, 30), Color.Black);
			
			//spriteBatch.DrawString(font, String.Format("Variables: {0}", phenotype.Variables.Serialize()), new Vector2(10, 460), Color.Black);

			int padding = 10;
			int offset = padding;
			foreach (KeyValuePair<CASettings.Setting, string> kvPair in menu.Settings.Dictionary())
			{
				spriteBatch.DrawString(font, kvPair.Key.ToString(), new Vector2(offset, 50), menu.Current == kvPair.Key ? Color.Red : Color.Black);
				spriteBatch.DrawString(font, kvPair.Value, new Vector2(offset, 70), Color.Black);
				offset += kvPair.Key.ToString().Length * 10 + padding;
			}

			spriteBatch.End();

			base.Draw(gameTime);
		}

	}
}
