﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AForge.Math;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace GaCa
{

	public class Size
	{
		public int Height { get; set; }
		public int Width { get; set; }

		public Size(int width, int height)
		{
			Width = width;
			Height = height;
		}
	}

	public class Position
	{
		public int X { get; set; }
		public int Y { get; set; }

		public Position(int x, int y)
		{
			X = x;
			Y = y;
		}

		public override string ToString()
		{
			return String.Format("({0}, {1})", X, Y);
		}
	}

	public static class Tools
	{

		public static Random Random { get; set; }

		public static List<string> Multiply(this List<string> list, int multiplier)
		{
			List<string> copy = list.ToArray().ToList();
			for (int i = 0; i < multiplier; i++)
				copy.AddRange(list);
			return copy;
		}

		public static char Flip(this char c, int numbase)
		{
			int oldbit = (int)Char.GetNumericValue(c);
			int add = (Random.Next(2) == 0 ? -1 : 1);
			int newbit = (oldbit + numbase + add) % numbase;
			////if (newNum == numbase) newNum = 0;
			return newbit.ToString()[0];
		}

		public static ulong Pow(int a, int b)
		{
			ulong result = 1;
			for (int i = 0; i < b; i++)
				result = result * (ulong)a;
			return result;
		}

		public static List<string> WithReturn(this List<string> list, List<string> range)
		{
			list.AddRange(range);
			return list;
		}

		public static string DecimalToBase(int iDec, int numbase, int length)
		{
			string strBin = "";
			int[] result = new int[length];
			int MaxBit = length;
			for (; iDec > 0; iDec /= numbase)
			{
				int rem = iDec % numbase;
				result[--MaxBit] = rem;
			}
			for (int i = 0; i < result.Length; i++)
				strBin += result.GetValue(i);
			return strBin;
		}

		public static ulong BaseToDecimal(string sBase, int numbase)
		{
			ulong dec = 0;
			ulong b;
			ulong iProduct = 1;
			//string sHexa = "";
			//if (numbase > base10)
			//    for (int i = 0; i < cHexa.Length; i++)
			//        sHexa += cHexa.GetValue(i).ToString();
			for (int i = sBase.Length - 1; i >= 0; i--, iProduct *= (ulong)numbase)
			{
				string sValue = sBase[i].ToString();
				b = (ulong)Char.GetNumericValue(sBase[i]);
				dec += (ulong)(b * iProduct);
			}
			return dec;
		}

		public static Complex[] ToComplexes(List<int> list)
		{
			List<Complex> complexes = new List<Complex>();
			foreach (int j in list)
				complexes.Add(new Complex(j, 0));
			return complexes.ToArray();
		}

		public static List<int> ToMagnitudes(Complex[] complexes)
		{
			List<int> list = new List<int>();
			foreach (Complex c in complexes)
				list.Add((int)c.Magnitude);
			return list;
		}

		public static void DrawPoint(this SpriteBatch spriteBatch, Color color, Vector2 point)
		{
			LineBatch.DrawLine(spriteBatch, color, point, new Vector2(point.X + 1, point.Y + 1));
		}

		public static void DrawLine(this SpriteBatch spriteBatch, Color color, Vector2 point1, Vector2 point2)
		{
			LineBatch.DrawLine(spriteBatch, color, point1, point2);
		}

		public static string Serialize(this List<double> list)
		{
			StringBuilder sb = new StringBuilder();
			foreach (double d in list)
				sb.Append(d).Append(" ");
			return sb.ToString();
		}

	}

	static public class LineBatch
	{
		static private Texture2D _empty_texture;
		static private bool _set_data = false;

		static public void Init(GraphicsDevice device)
		{
			_empty_texture = new Texture2D(device, 1, 1, true, SurfaceFormat.Color);
		}

		static public void DrawLine(SpriteBatch batch, Color color,
									Vector2 point1, Vector2 point2)
		{

			DrawLine(batch, color, point1, point2, 0);
		}

		/// <summary>
		/// Draw a line into a SpriteBatch
		/// </summary>
		/// <param name="batch">SpriteBatch to draw line</param>
		/// <param name="color">The line color</param>
		/// <param name="point1">Start Point</param>
		/// <param name="point2">End Point</param>
		/// <param name="Layer">Layer or Z position</param>
		static public void DrawLine(SpriteBatch batch, Color color, Vector2 point1,
									Vector2 point2, float Layer)
		{
			//Check if data has been set for texture
			//Do this only once otherwise
			if (!_set_data)
			{
				_empty_texture.SetData(new[] { Color.White });
				_set_data = true;
			}


			float angle = (float)Math.Atan2(point2.Y - point1.Y, point2.X - point1.X);
			float length = (point2 - point1).Length();

			batch.Draw(_empty_texture, point1, null, color,
					   angle, Vector2.Zero, new Vector2(length, 1),
					   SpriteEffects.None, Layer);
		}
	}

}
